import abc
import logging

import six

from contextlib import contextmanager

from ..loggingadapter import LogIdAdapter

_logger = logging.getLogger(__name__)


@six.add_metaclass(abc.ABCMeta)
class AbstractBackend:
    _connection_params = None

    @abc.abstractmethod
    def _connect(self):
        """Connects to the backend and returns a connection."""

    def execute(self, stream=False, *args, **kwargs):
        """Executes the query and returns the result."""


@contextmanager
def get_connection(backend, log_id=None):
    """Returns a DB connection."""
    adapter = LogIdAdapter(_logger, dict(log_id=log_id))

    try:
        adapter.info("Connecting to DB")
        connection = backend._connect()
    except:
        adapter.exception("Cannot connect to DB")
        raise

    try:
        yield connection
    finally:
        try:
            adapter.info("Closed connection to DB")
            connection.close()
        except:
            pass



import csv
import getpass
import logging
import os
import unittest

from rapyd_db.utils import _get_uuid
from rapyd_db.backends import get_connection
from rapyd_db.backends.mysql import MySQL


logging.basicConfig(level="WARNING")


class TestMySQLBackend(unittest.TestCase):
    def setUp(self):
        # read connection params from environment variables
        self._host = os.environ.get("MYSQL_HOST") or "localhost"
        self._port = os.environ.get("MYSQL_PORT") or 3306
        self._user = os.environ.get("MYSQL_USER") or getpass.getuser()
        self._password = os.environ.get("MYSQL_PASSWORD")
        self._test_db = os.environ.get("MYSQL_TEST_DB") or "test_db"

        self._db = MySQL(
            host=self._host, user=self._user, password=self._password, port=self._port,
        )

    def test_00_mysql_db_connection(self):
        rows_affected, last_row_id, rows = self._db.execute(
            "SELECT version()", stream=False
        )
        self.assertIn("version()", rows[0])

    def test_01_mysql_create_test_db(self):
        rows_affected, last_row_id, rows = self._db.execute(
            "CREATE DATABASE IF NOT EXISTS `{}`".format(self._test_db), stream=False
        )
        self.assertEqual(rows_affected, 1)

    def test_02_mysql_create_salaries_table(self):
        query = (
            "CREATE TABLE IF NOT EXISTS `{}`.`salaries` ("
            " `emp_no` int(11) NOT NULL,"
            " `salary` int(11) NOT NULL,"
            " `from_date` date NOT NULL,"
            " `to_date` date NOT NULL,"
            " PRIMARY KEY (`emp_no`,`from_date`))".format(self._test_db)
        )
        self._db.execute(query, stream=False)

    def test_03_mysql_insert_salaries(self):
        data = []
        base_dir = os.path.dirname(os.path.abspath(__file__))
        with open(os.path.join(base_dir, "test_data_salaries.csv")) as data_file:
            reader = csv.DictReader(data_file)
            for row in reader:
                data.append(row.values())
        self.assertEqual(1000, len(data))
        query = (
            "INSERT INTO `{}`.`salaries` (`emp_no`, `salary`, `from_date`, `to_date`)"
            " VALUES (%s, %s, %s, %s)".format(self._test_db)
        )
        with get_connection(self._db, _get_uuid()) as connection:
            connection.autocommit(False)
            cursor = connection.cursor()
            cursor.executemany(query, data)
            connection.commit()

    def test_04_mysql_stream_salaries(self):
        query = "SELECT * FROM {}.`salaries`".format(self._test_db)
        rows = self._db.execute(query, stream=True)
        count = 0
        for row in rows:
            count += 1
        self.assertEqual(1000, count)

    def test_99_mysql_delete_test_db(self):
        rows_affected, last_row_id, rows = self._db.execute(
            "DROP DATABASE IF EXISTS `{}`".format(self._test_db)
        )
        self.assertEqual(rows_affected, 1)


if __name__ == "__main__":
    unittest.main()



import logging
import MySQLdb

from datetime import datetime
from MySQLdb.cursors import DictCursor, SSDictCursor

from . import AbstractBackend, get_connection
from ..loggingadapter import LogIdAdapter
from ..utils import _assign_if_not_none, _get_uuid


_logger = logging.getLogger(__name__)


class MySQL(AbstractBackend):
    def __init__(self, host=None, user=None, password=None, **kwargs):
        """
        Initializes an instance of the MySQL backend with the connection parameters.

        :param str host: Name of the host to connect to.
        :param str user: User to authenticate as.
        :param str password: Password to authenticate with.
        :param kwargs:
            All other parameters supported by the MySQLdb `connect()` method.
            Refer https://mysqlclient.readthedocs.io/user_guide.html#functions-and-attributes for additional examples.
            Note: `cursorclass` is limited to return dictionaries only and cannot be changed.
        """
        self._connection_params = dict()
        _assign_if_not_none(self._connection_params, "host", host)
        _assign_if_not_none(self._connection_params, "user", user)
        _assign_if_not_none(self._connection_params, "password", password)
        self._connection_params.update(kwargs)
        # we will remove cursor class from as this will be set in the underlying methods
        self._connection_params.pop("cursorclass", None)

    def _connect(self):
        return MySQLdb.connect(**self._connection_params)

    def execute(self, query, params=None, stream=False):
        """
        Executes the query and returns the result.

        :param str query: The query to execute.
        :param tuple params: A tuple of parameters for substitution prior to executing the query.
        :param bool stream:
            When `True`, a generator is returned which will fetch data from the
            DB in a lazy fashion. Typically used when you want to
            return large volumes of data from the DB while while avoiding `MemoryError`.
        :return:
            Returns a generator when `stream` is `True`. Otherwise returns a
            tuple of the rows affected and a list of all rows returned after
            query execution.
        """
        # the return has to be done this way to accommodate having
        # `yield` and `return` in the same method
        # https://stackoverflow.com/a/43459115/399435
        # unfortunately there is a lot of code duplication here
        if stream:
            # when streaming, we want to keep results on the server side to reduce client side memory footprint
            self._connection_params["cursorclass"] = SSDictCursor
            return self._stream(query, params)
        else:
            self._connection_params["cursorclass"] = DictCursor
            return self._no_stream(query, params)

    def _stream(self, query, params):
        # setup logging
        log_id = _get_uuid()
        adapter = LogIdAdapter(_logger, dict(log_id=log_id))

        with get_connection(self, log_id) as connection:
            connection.autocommit(True)
            cursor = connection.cursor()
            execution_start = datetime.now()
            adapter.info("Starting executing query at {}".format(execution_start))
            adapter.info("Streaming results from DB.")

            if params is not None:
                cursor.execute(query, params)
            else:
                cursor.execute(query)

            adapter.info("{}".format(cursor._last_executed.decode("utf8")))

            # returns the generator object
            for row in cursor:
                yield row

            execution_end = datetime.now()
            adapter.info(
                "Executed in {} second(s)".format(
                    (execution_end - execution_start).seconds
                )
            )
            adapter.info("Ended query execution at {}".format(execution_end))

    def _no_stream(self, query, params):
        # setup logging
        log_id = _get_uuid()
        adapter = LogIdAdapter(_logger, dict(log_id=log_id))

        with get_connection(self, log_id) as connection:
            connection.autocommit(True)
            cursor = connection.cursor()
            execution_start = datetime.now()
            adapter.info("Starting executing query at {}".format(execution_start))
            adapter.info("Not streaming results from DB.")

            if params is not None:
                rows_affected = cursor.execute(query, params)
            else:
                rows_affected = cursor.execute(query)

            execution_end = datetime.now()
            adapter.info("{}".format(cursor._last_executed.decode("utf8")))
            adapter.info(
                "{} row(s) affected in {} second(s)".format(
                    rows_affected, (execution_end - execution_start).seconds
                )
            )
            adapter.info("Ended query execution at {}".format(execution_end))

            # returns rows affected and all results
            return rows_affected, cursor.lastrowid, cursor.fetchall()



import csv
import getpass
import itertools
import logging
import os
import unittest

from rapyd_db.utils import _get_uuid
from rapyd_db.backends import get_connection
from rapyd_db.backends.mssql import MSSQL

logging.basicConfig(level="WARNING")


class TestMSSQLBackend(unittest.TestCase):
    def setUp(self):
        # read connection params from environment variables
        self._host = os.environ.get("MSSQL_HOST") or "localhost"
        self._port = os.environ.get("MSSQL_PORT") or 1433
        self._user = os.environ.get("MSSQL_USER") or getpass.getuser()
        self._password = os.environ.get("MSSQL_PASSWORD")
        self._test_db = os.environ.get("MSSQL_TEST_DB") or "test_db"

        self._db = MSSQL(
            host=self._host, user=self._user, password=self._password, port=self._port,
        )

    def test_00_mssql_db_connection(self):
        rows_affected, last_row_id, rows = self._db.execute(
            "SELECT @@VERSION AS 'version'", stream=False
        )
        self.assertIn("version", rows[0])

    def test_01_mssql_create_test_db(self):
        rows_affected, last_row_id, rows = self._db.execute(
            "CREATE DATABASE [{}]".format(self._test_db), stream=False
        )

    def test_02_mssql_create_salaries_table(self):
        query = (
            "CREATE TABLE [{}].[dbo].[salaries] ("
            " [emp_no] int NOT NULL,"
            " [salary] int NOT NULL,"
            " [from_date] date NOT NULL,"
            " [to_date] date NOT NULL,"
            " PRIMARY KEY ([emp_no],[from_date]))".format(self._test_db)
        )
        self._db.execute(query, stream=False)

    def test_03_mssql_insert_salaries(self):
        data = []
        base_dir = os.path.dirname(os.path.abspath(__file__))
        with open(os.path.join(base_dir, "test_data_salaries.csv")) as data_file:
            reader = csv.DictReader(data_file)
            for row in reader:
                data.append(tuple(row.values()))
        self.assertEqual(1000, len(data))
        query = (
            "INSERT INTO [{}].[dbo].[salaries] ([emp_no], [salary], [from_date], [to_date])"
            " VALUES (%s, %s, %s, %s)".format(self._test_db)
        )
        with get_connection(self._db, _get_uuid()) as connection:
            connection.autocommit(False)
            cursor = connection.cursor()
            cursor.executemany(query, data)
            connection.commit()

    def test_04_mssql_stream_salaries(self):
        query = "SELECT * FROM [{}].[dbo].[salaries]".format(self._test_db)
        rows = self._db.execute(query, stream=True)
        count = 0
        for row in rows:
            count += 1
        self.assertEqual(1000, count)

    def test_99_mssql_delete_test_db(self):
        rows_affected, last_row_id, rows = self._db.execute(
            "DROP DATABASE [{}]".format(self._test_db)
        )


if __name__ == "__main__":
    unittest.main()



import logging

from datetime import datetime
from pymongo import MongoClient

from . import AbstractBackend, get_connection
from ..loggingadapter import LogIdAdapter
from ..utils import _assign_if_not_none, _get_uuid


_logger = logging.getLogger(__name__)


class Mongo(AbstractBackend):
    def __init__(
        self,
        host=None,
        username=None,
        password=None,
        auth_source="admin",
        connect_timeout_ms=2000,
        **kwargs
    ):
        """
        Initializes an instance of the Mongo backend with the connection parameters.

        :param str host: Can be a full mongoDB URI or a simple hostname.
        :param str username: User to authenticate as.
        :param str password: Password to authenticate with.
        :param str auth_source: Database to authenticate against. Defaults to admin.
        :param int connect_timeout_ms:
            How long to wait when connecting to server before concluding server is unavailable.
            Defaults to 2000 (2 seconds).
        :param kwargs:
            All other parameters supported by the MongoClient `__init__()` method.
            Refer https://api.mongodb.com/python/current/api/pymongo/mongo_client.html for additional examples.
            Note: `maxPoolSize` is set to 1 only and cannot be changed.
            Note: `connect` is also set to False because a connection should only occur while querying
        """
        self._connection_params = dict()
        _assign_if_not_none(self._connection_params, "host", host)
        _assign_if_not_none(self._connection_params, "username", username)
        _assign_if_not_none(self._connection_params, "password", password)
        _assign_if_not_none(self._connection_params, "authSource", auth_source)
        _assign_if_not_none(
            self._connection_params, "connectTimeoutMS", connect_timeout_ms
        )
        self._connection_params.update(kwargs)
        self._connection_params["connect"] = False
        self._connection_params["maxPoolSize"] = 1

    def _connect(self):
        return MongoClient(**self._connection_params)

    def execute(self, operation, *args, **kwargs):
        """
        Executes the query and returns the result.

        :param str operation: Operation to run.
        :param str database: Database to use.
        :param str collection: Collection to use.
        :param bool stream:
            When `True`, a generator is returned which will fetch data from the
            DB in a lazy fashion. Typically used when you want to
            return large volumes of data from the DB while while avoiding `MemoryError`.
            Parameters `database` and `collection` are required when `stream=True`.
        :param args:
            All other positional arguments supported by the method you are calling via operation.
        :param kwargs:
            All other keyword arguments supported by the method you are calling via operation.
        :return:
            Returns a generator when `stream` is `True`. Otherwise returns a
            the result of the method you are calling via operation.
        """
        # in python 2 default arguments cannot be used with args and kwargs
        # https://stackoverflow.com/a/15302038/399435
        # so doing it this way
        stream = kwargs.pop("stream", False)

        # the return has to be done this way to accommodate having
        # `yield` and `return` in the same method
        # https://stackoverflow.com/a/43459115/399435
        # unfortunately there is a lot of code duplication here
        if stream:
            # when streaming, we want to keep results on the server side to reduce client side memory footprint
            return self._stream(operation, *args, **kwargs)
        else:
            return self._no_stream(operation, *args, **kwargs)

    def _stream(self, operation, *args, **kwargs):
        # setup logging
        log_id = _get_uuid()
        adapter = LogIdAdapter(_logger, dict(log_id=log_id))

        # get some optional parms if present
        database = kwargs.pop("database", None)
        collection = kwargs.pop("collection", None)
        if database is None or collection is None:
            raise KeyError(
                "Parameters 'database' and 'collection' are required when stream=True"
            )

        with get_connection(self, log_id) as connection:
            execution_start = datetime.now()
            adapter.info("Using database {}".format(database))
            adapter.info("Using collection {}".format(collection))
            adapter.info("args: {}".format(args))
            adapter.info("kwargs: {}".format(kwargs))
            adapter.info(
                "Started executing {} at {}".format(operation, execution_start)
            )
            adapter.info("Streaming results from DB.")
            operation_callable = getattr(connection[database][collection], operation)
            result = operation_callable(*args, **kwargs)

            # returns the generator object
            for row in result:
                yield row

            execution_end = datetime.now()
            adapter.info(
                "Executed in {} second(s)".format(
                    (execution_end - execution_start).seconds
                )
            )
            adapter.info("Ended {} execution at {}".format(operation, execution_end))

    def _no_stream(self, operation, *args, **kwargs):
        # setup logging
        log_id = _get_uuid()
        adapter = LogIdAdapter(_logger, dict(log_id=log_id))

        # get some optional parms if present
        database = kwargs.pop("database", None)
        collection = kwargs.pop("collection", None)

        with get_connection(self, log_id) as connection:
            execution_start = datetime.now()
            if database is not None:
                adapter.info("Using database {}".format(database))
            if collection is not None:
                adapter.info("Using collection {}".format(collection))
            adapter.info("args: {}".format(args))
            adapter.info("kwargs: {}".format(kwargs))
            adapter.info(
                "Started executing {} at {}".format(operation, execution_start)
            )
            adapter.info("Not streaming results from DB.")

            if database is not None and collection is not None:
                operation_callable = getattr(
                    connection[database][collection], operation
                )
            elif database is not None and collection is None:
                operation_callable = getattr(connection[database], operation)
            else:
                operation_callable = getattr(connection, operation)
            result = operation_callable(*args, **kwargs)

            execution_end = datetime.now()
            adapter.info(
                "Executed in {} second(s)".format(
                    (execution_end - execution_start).seconds
                )
            )
            adapter.info("Ended {} execution at {}".format(operation, execution_end))
            return result



import uuid


def _assign_if_not_none(obj, param, value):
    """A method to quickly assign a value if it is not none to either a dictionary or an object."""
    if value:
        if isinstance(obj, dict):
            obj[param] = value
        else:
            setattr(obj, param, value)
        return True
    return False


def _get_uuid():
    """Returns a unique ID which can be used to track log messages by query."""
    return uuid.uuid4().hex



import os
from setuptools import setup, find_packages

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

with open("README.rst") as f:
    readme = f.read()

with open("LICENSE") as f:
    license = f.read()

setup(
    name="rapyd_db",
    version="0.0.7",
    description="An opinionated lightweight wrapper around various DB backend drivers.",
    long_description=readme,
    author="Karthic Raghupathi",
    author_email="karthicr@gmail.com",
    url="https://github.com/karthicraghupathi/rapyd_db",
    license=license,
    packages=find_packages(exclude=("tests", "docs")),
    install_requires=["Cython", "six",],
    extras_require={
        "mysql": ["mysqlclient"],
        "mongo": ["pymongo"],
        "mssql": ["pymssql"],
    },
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
    ],
)



import logging


class LogIdAdapter(logging.LoggerAdapter):
    """
    This adapter will look to see if there is a unique log record identifier.
    If present, it will log that value along with the message.
    This is used to tie log messages performing a unit of work for easy audits.
    """

    def process(self, msg, kwargs):
        log_id = self.extra.get("log_id")
        if log_id:
            return "{} - {}".format(log_id, msg), kwargs
        else:
            return msg, kwargs



import csv
import getpass
import logging
import os
import unittest

from rapyd_db.backends.mongo import Mongo


logging.basicConfig(level="WARNING")


class TestMongoBackend(unittest.TestCase):
    def setUp(self):
        # read connection params from environment variables
        self._host = os.environ.get("MONGO_HOST") or "localhost"
        self._port = os.environ.get("MONGO_PORT") or 27017
        self._username = os.environ.get("MONGO_USERNAME") or getpass.getuser()
        self._password = os.environ.get("MONGO_PASSWORD")
        self._test_db = os.environ.get("MONGO_TEST_DB") or "test_db"
        self._test_collection = (
            os.environ.get("MONGO_TEST_COLLECTION") or "test_collection"
        )
        self._db = Mongo(
            host=self._host,
            username=self._username,
            password=self._password,
            port=self._port,
        )

    def test_00_mongo_db_connection(self):
        result = self._db.execute("server_info")
        self.assertIn("version", result)

    def test_03_mongo_insert_salaries(self):
        data = []
        base_dir = os.path.dirname(os.path.abspath(__file__))
        with open(os.path.join(base_dir, "test_data_salaries.csv")) as data_file:
            reader = csv.DictReader(data_file)
            for row in reader:
                data.append(row)
        self.assertEqual(1000, len(data))
        self._db.execute(
            "insert_many",
            data,
            database=self._test_db,
            collection=self._test_collection,
        )

    def test_04_mongo_stream_salaries(self):
        rows = self._db.execute(
            "find",
            {},
            database=self._test_db,
            collection=self._test_collection,
            stream=True,
        )
        count = 0
        for row in rows:
            count += 1
        self.assertEqual(1000, count)

    def test_99_mongo_delete_test_db(self):
        self._db.execute("drop_database", self._test_db)


if __name__ == "__main__":
    unittest.main()
